/* attract.c - strange attractors etc. */

#include <math.h>
#include "attract.h"

#define MAXN 3

/* Runge-Kutta method */
void
rk(double step, int n, rk_func_t func, void *data, double t, double *y0, double *y1)
{
  double k1[MAXN],k2[MAXN],k3[MAXN],k4[MAXN],tmp[MAXN];
  int i;

  func(data,t,y0,k1);
  for (i=0; i<n; i++) tmp[i] = y0[i]+step/2*k1[i];
  func(data,t+step/2,tmp,k2);
  for (i=0; i<n; i++) tmp[i] = y0[i]+step/2*k2[i];
  func(data,t+step/2,tmp,k3);
  for (i=0; i<n; i++) tmp[i] = y0[i]+step*k3[i];
  func(data,t+step,tmp,k4);
  
  for (i=0; i<n; i++)
    y1[i] = y0[i] + step/6*(k1[i]+2*k2[i]+2*k3[i]+k4[i]);
}

void
lorenz(void *data, double t, double *y, double *dx)
{
  double *c = data;
  
  (void)t;
  
  dx[0] = c[0]*(y[1]-y[0]);
  dx[1] = y[0]*(c[1]-y[2]) - y[1];
  dx[2] = y[0]*y[1] - c[2]*y[2];
}

void
pickover(double *p, double *x, double *y)
{
  y[0] = sin(p[0]*x[1]) - x[2]*cos(p[1]*x[0]);
  y[1] = x[2]*sin(p[2]*x[0]) - cos(p[3]*x[1]);
  y[2] = sin(x[0]);
}
