/* convolve.c - image convolution */

#include <assert.h>
#include "im.h"
#include "xmalloc.h"

#define convolve(s,d,m,w,h,c,u,v)       \
{                                       \
  int a, b, i, k, l;                    \
  void *p = m;                          \
  for (a = 0; a < h; a++)               \
    for (b = 0; b < w; b++)             \
      for (i = 0; i < c; i++)           \
        {                               \
          m = p;                        \
          *d = 0;                       \
          for (k = 0; k < v; k++)       \
            for (l = 0; l < u; l++)     \
              *d += s[(k*w+l)*c]**m++;  \
          *d>>=8;                       \
          d++;                          \
          s++;                          \
        }                               \
}

#define convolve_x(s,d,m,w,h,n)         \
{                                       \
  int a, b, i;                          \
  int *p, *q;                           \
  for (a = 0; a < h; a++)               \
    for (b = 0; b < w; b++)             \
      {                                 \
        p = m;                          \
        q = s;                          \
        *d=0;                           \
        for (i = 0; i < n; i++)         \
          *d += *p++**q++;              \
        *d>>=8;                         \
        d++;                            \
        s++;                            \
      }                                 \
}

#define convolve_y(s,d,m,w,h,n)         \
{                                       \
  int a, b, i;                          \
  int *p;                               \
  for (a = 0; a < h; a++)               \
    for (b = 0; b < w; b++)             \
      {                                 \
        p = m;                          \
        *d=0;                           \
        for (i = 0; i < n; i++)         \
          *d += s[(a+i)*w+b]**p++;      \
        *d>>=8;                         \
        d++;                            \
      }                                 \
}

#define convolve_func(n)                                                       \
int *                                                                          \
convolve_ ## n ## x ## n(int *s, int *d, int *m, int w, int h, int c)          \
{                                                                              \
  int *tmp = d;                                                                \
  convolve(s,d,m,w,h,c,n,n);                                                   \
  return tmp;                                                                  \
}

#define convolve_x_func(n)                                                     \
int *                                                                          \
convolve_x_ ## n(int *s, int *d, int *m, int w, int h, int c)                  \
{                                                                              \
  int *tmp = d;                                                                \
  (void)c;                                                                     \
  convolve_x(s,d,m,w,h-1,n);                                                   \
  return tmp;                                                                  \
}

#define convolve_y_func(n)                                                     \
int *                                                                          \
convolve_y_ ## n(int *s, int *d, int *m, int w, int h, int c)                  \
{                                                                              \
  int *tmp = d;                                                                \
  (void)c;                                                                     \
  convolve_y(s,d,m,w,h-n,n);                                                   \
  return tmp;                                                                  \
}

#define im_convolve_func(n)                              \
im_t *                                                   \
im_convolve_ ## n ## x ## n(im_t *s, im_t *d, int *m)    \
{                                                        \
  int *sp, *dp;                                          \
  int w, h, c;                                           \
                                                         \
  if (d == NULL)                                         \
    d = im_new(s->w, s->h, s->channels);                 \
                                                         \
  sp = s->data;                                          \
  dp = d->data;                                          \
  w = s->w;                                              \
  h = s->h;                                              \
  c = s->channels;                                       \
  convolve_ ## n ## x ## n(sp,dp,m,w,h,c);               \
                                                         \
  return d;                                              \
}

#define im_convolve_x_func(n)                            \
im_t *                                                   \
im_convolve_x_ ## n(im_t *s, im_t *d, int *m)            \
{                                                        \
  int *sp, *dp;                                          \
  int w, h, c;                                           \
                                                         \
  if (d == NULL)                                         \
    d = im_new(s->w, s->h, s->channels);                 \
                                                         \
  sp = s->data;                                          \
  dp = d->data;                                          \
  w = s->w;                                              \
  h = s->h;                                              \
  c = s->channels;                                       \
  convolve_x_ ## n(sp,dp,m,w,h,c);                       \
                                                         \
  return d;                                              \
}

#define im_convolve_y_func(n)                            \
im_t *                                                   \
im_convolve_y_ ## n(im_t *s, im_t *d, int *m)            \
{                                                        \
  int *sp, *dp;                                          \
  int w, h, c;                                           \
                                                         \
  if (d == NULL)                                         \
    d = im_new(s->w, s->h, s->channels);                 \
                                                         \
  sp = s->data;                                          \
  dp = d->data;                                          \
  w = s->w;                                              \
  h = s->h;                                              \
  c = s->channels;                                       \
  convolve_y_ ## n(sp,dp,m,w,h,c);                       \
                                                         \
  return d;                                              \
}

#define im_convolve_separable_func(w,h)                                         \
im_t *                                                                          \
im_convolve_separable_ ## w ## x ## h(im_t *s, im_t *d, int *mx, int *my)       \
{                                                                               \
  im_t *t = im_convolve_x_ ## w(s, NULL, mx);                                   \
  d = im_convolve_y_ ## h(t, d, my);                                            \
  im_free(t);                                                                   \
  return d;                                                                     \
}

#if 0
im_t *
im_convolve(im_t *s, im_t *d, int *m, int nx, int ny)
{
  (void)ny;

  switch (nx)
    {
    case  2: return im_convolve_2x2(s,d,m);
    case  3: return im_convolve_3x3(s,d,m);
    case  9: return im_convolve_9x9(s,d,m);
    case 13: return im_convolve_13x13(s,d,m);
    case 15: return im_convolve_15x15(s,d,m);
    case 17: return im_convolve_17x17(s,d,m);
    case 25: return im_convolve_25x25(s,d,m);
    default:
      fprintf(stderr, "im_convolve(): %dx%d convolution not supported\n", nx, ny);
      exit(1);
    }

  /* we should not reach here */
  return NULL;
}

im_t *
im_convolve_separable(im_t *s, im_t *d, int *mx, int *my, int nx, int ny)
{
  (void)ny;

  switch (nx)
    {
    case  3: return im_convolve_separable_3x3(s,d,mx,my);
    case  9: return im_convolve_separable_9x9(s,d,mx,my);
    case 15: return im_convolve_separable_15x15(s,d,mx,my);
    case 25: return im_convolve_separable_25x25(s,d,mx,my);
    case 35: return im_convolve_separable_35x35(s,d,mx,my);
    case 55: return im_convolve_separable_55x55(s,d,mx,my);
    default:
      fprintf(stderr, "im_convolve_separable(): %dx%d convolution not supported\n", nx, ny);
      exit(1);
    }

  /* we should not reach here */
  return NULL;
}
#endif

convolve_x_func(3)
convolve_x_func(5)
convolve_y_func(3)
convolve_y_func(5)

im_convolve_x_func(3)
im_convolve_x_func(5)
im_convolve_y_func(3)
im_convolve_y_func(5)

im_convolve_separable_func(3,3)
im_convolve_separable_func(5,5)
