/* editor.c */

#include <SDL.h>
#include <math.h>
#include "main.h"
#include "script.h"

#define PICS 14

enum
{
  DEC_TIME=0,INC_TIME,
  DEC_DURATION,INC_DURATION,
  DEC_SCALEX,INC_SCALEX,
  DEC_SCALEY,INC_SCALEY,
  DEC_ROTATION,INC_ROTATION,
  KEYS
};

/* function prototypes */
void shutdown(void);
void process_events(void);
int mkint(int *, int);
int getparam(void);

/* variables */
extern im_t *pics[SCRIPT_MAX_PICS];
extern int pics_n;
int metaframes;
im_t *imbuf;
int keys[KEYS]={0},param[10],param_n,step,lastd;
script_t *current;
double amount;


void shutdown(void) { SDL_Quit(); }

int
mkint(int *a, int n)
{
  int c=1,v=0;

  while (n--)
    {
      v += c*a[n];
      c *= 10;
    }
  return v;
}

int
getparam(void)
{
  int v = mkint(param,param_n);
  param_n=0;
  return v;
} 

int
main(int argc, char *argv[])
{
  SDL_Surface *surface;
  SDL_Event event;
  Uint32 last, t;
  char name[50];
  int num, playing, tmpticks,tmp;
  FILE *f;
  
  if (argc < 2) return 0;
  
  /* load pictures */
  for (pics_n=0; pics_n<PICS; pics_n++)
    {
      sprintf(name, "data/pic%d.pgm", pics_n);
      if ((f=fopen(name, "r")) == NULL)
        {
          fprintf(stderr, "error: couldn't open '%s'\n", name);
          return 1;
        }
      pics[pics_n] = im_new_rotozoom(pnm_read(f));
      fclose(f);
    }

  /*...*/
  script_init();
  imbuf = im_new(BUF_WIDTH,BUF_HEIGHT,1);

  /* initialize video and audio */
  if (SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO | SDL_INIT_AUDIO))
    {
      fprintf(stderr, "error: couldn't initialize SDL: %s\n", SDL_GetError());
      return 1;
    }
  atexit(shutdown);

  if ((surface=SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 24, 0)) == NULL)
    return 1;

  /* main loop */
  param_n = 0;
  current = NULL;
  playing = tmpticks = 0;
  amount = 0.1;
  step = 1;
  lastd = 50;
  
  metaframes = 0;
  last = SDL_GetTicks();

  while (1)
    {
      /* update metaframe counter (100hz) */
      t = SDL_GetTicks();
      if (t-last >= 10)
        while (last<t)
          {
            last += 10;
            metaframes++;
            if (playing) script_advance(1);
          }
      
      /* draw the next frame (TODO: limit framerate) */
      script_draw(imbuf);
      SDL_LockSurface(surface);
      script_render(imbuf, surface->pixels);
      SDL_UnlockSurface(surface);
      SDL_Flip(surface);

      /* check for input events */
      while (SDL_PollEvent(&event))
        switch (event.type)
          {
          case SDL_KEYDOWN:
            switch (event.key.keysym.sym)
              {
              case SDLK_0:
              case SDLK_1:
              case SDLK_2:
              case SDLK_3:
              case SDLK_4:
              case SDLK_5:
              case SDLK_6:
              case SDLK_7:
              case SDLK_8:
              case SDLK_9:
                num = event.key.keysym.sym-SDLK_0;
                if (param_n<9)
                  param[param_n++] = num;
                break;
              
              case SDLK_ESCAPE:
                exit(0);
              case SDLK_p:
                current = script_add();
                current->p = getparam()%PICS;
                current->d = lastd;
                break;
              case SDLK_u:
                current = script_add();
                current->p = getparam()%PICS;
                current->d = lastd;
                current->f = 0;
                break;
              case SDLK_o:
                script_remove();
                current=NULL;
                break;
              case SDLK_l:
                if (playing)
                  {
                    playing = 0;
                    script_set_ticks(tmpticks);
                  }
                else
                  {
                    playing = 1;
                    tmpticks = script_advance(0);
                    script_set_ticks(0);
                  }
                break;
              case SDLK_k:
                if (playing)
                  {
                    playing = 0;
                    script_set_ticks(tmpticks);
                  }
                else
                  {
                    playing = 1;
                    tmpticks = script_advance(0);
                  }
                break;

              case SDLK_m:
                if ((tmp=getparam()))
                  step = tmp;
                fprintf(stderr, "step=%d\n", step);
                break;

              case SDLK_SPACE:
                if (current)
                  current = NULL;
                else
                  {
                    if ((current = script_get()))
                      fprintf(stderr, "current: p=%d,t=%d,d=%d,x=%d,y=%d,a=%.3f,u=%.3f,v=%.3f\n",current->p,current->t,current->d,current->x,current->y,current->a,current->u,current->v);
                  }
                break;

              case SDLK_LESS:
                {
                  script_t *tmp = script_rotate();
                  if (tmp)
                    fprintf(stderr, "current: p=%d,t=%d,d=%d,x=%d,y=%d,a=%.3f,u=%.3f,v=%.3f\n",tmp->p,tmp->t,tmp->d,tmp->x,tmp->y,tmp->a,tmp->u,tmp->v);
                }
                break;

              case SDLK_MINUS:
                if (current)
                  current->p = getparam()%PICS;
                break;

              case SDLK_COMMA: script_load(argv[1]); break;
              case SDLK_PERIOD: script_save(argv[1]); break;

              case SDLK_q: keys[DEC_SCALEX]=keys[DEC_SCALEY]=1; break;
              case SDLK_w: keys[INC_SCALEX]=keys[INC_SCALEY]=1; break;
              case SDLK_e: keys[DEC_SCALEX]=1; break;
              case SDLK_r: keys[INC_SCALEX]=1; break;
              case SDLK_t: keys[DEC_SCALEY]=1; break;
              case SDLK_y: keys[INC_SCALEY]=1; break;
              case SDLK_a: keys[DEC_ROTATION]=1; break;
              case SDLK_s: keys[INC_ROTATION]=1; break;
              case SDLK_z: keys[DEC_DURATION]=1; break;
              case SDLK_x: keys[INC_DURATION]=1; break;
              case SDLK_LEFT: keys[DEC_TIME]=1; break;
              case SDLK_RIGHT: keys[INC_TIME]=1; break;
              
              default: break;
              }
            break;

          case SDL_KEYUP:
            switch (event.key.keysym.sym)
              {
              case SDLK_q: keys[DEC_SCALEX]=keys[DEC_SCALEY]=0;  break;
              case SDLK_w: keys[INC_SCALEX]=keys[INC_SCALEY]=0; break;
              case SDLK_e: keys[DEC_SCALEX]=0; break;
              case SDLK_r: keys[INC_SCALEX]=0; break;
              case SDLK_t: keys[DEC_SCALEY]=0; break;
              case SDLK_y: keys[INC_SCALEY]=0; break;
              case SDLK_a: keys[DEC_ROTATION]=0; break;
              case SDLK_s: keys[INC_ROTATION]=0; break;
              case SDLK_z: keys[DEC_DURATION]=0; if(current){fprintf(stderr,"d=%d\n",current->d);lastd=current->d;}break;
              case SDLK_x: keys[INC_DURATION]=0; if(current){fprintf(stderr,"d=%d\n",current->d);lastd=current->d;}break;
              case SDLK_LEFT: keys[DEC_TIME]=0; if(current)fprintf(stderr,"t=%d\n",current->t); else fprintf(stderr,"time=%d\n",script_advance(0));break;
              case SDLK_RIGHT: keys[INC_TIME]=0; if(current)fprintf(stderr,"t=%d\n",current->t); else fprintf(stderr,"time=%d\n",script_advance(0));break;

              default: break;
              }
            break;

          case SDL_MOUSEMOTION:
            if (current)
              {
                current->x += event.motion.xrel;
                current->y += event.motion.yrel;
              }
            break;

          case SDL_MOUSEBUTTONDOWN:
            switch (event.button.button)
              {
              case 1: keys[DEC_ROTATION]=1; break;
              case 3: keys[INC_ROTATION]=1; break;
              case 4: amount += 0.01; break;
              case 5: amount -= 0.01; break;
              }
            break;

          case SDL_MOUSEBUTTONUP:
            switch (event.button.button)
              {
              case 1: keys[DEC_ROTATION]=0; break;
              case 3: keys[INC_ROTATION]=0; break;
              }
            break;

          case SDL_QUIT: exit(0);
          }

      process_events();

//      fprintf(stderr, "%.2f\n", 1000.0/(double)(SDL_GetTicks()-t));
    }

  return 0;
}

void
process_events(void)
{
  if (keys[DEC_TIME])
    {
      script_advance(-step);
      script_update();
    }
  if (keys[INC_TIME])
    {
      script_advance(step);
      script_update();
    }

  if (!current) return;

  if (keys[DEC_TIME])
    current->t -= step;
  if (keys[INC_TIME])
    current->t += step;
  if (keys[DEC_DURATION])
    current->d -= step;
  if (keys[INC_DURATION])
    current->d += step;
  if (keys[DEC_SCALEX])
      current->u -= 0.015;
  if (keys[DEC_SCALEY])
      current->v -= 0.015;
  if (keys[INC_SCALEX])
      current->u += 0.015;
  if (keys[INC_SCALEY])
      current->v += 0.015;
  if (keys[DEC_ROTATION])
    current->a -= amount;
  if (keys[INC_ROTATION])
    current->a += amount;
}
