/* filter.c - filter kernels and methods */

#include <math.h>
#include "im.h"
#include "xmalloc.h"


/* calculate one dimensional gaussian kernel */
int *
filter_build_gaussian1d(int *kernel, double d, int n)
{
  double ker[100], step, x, sum;
  int i;
  
  step = 6.0/((double)(n-1));
  sum = 0;

  x = -3.0;
  for (i = 0; i < n; i++)
    {
      ker[i] = 1.0/(sqrt(2.0*M_PI)*d) * exp(-x*x/(2.0*d*d));
      sum += ker[i];
      x += step;
    }

  sum = 1.0/sum;
  for (i = 0; i < n; i++)
    kernel[i] = (int)(256.0*ker[i]*sum);

  return kernel;
}

#if 0
/* calculate two dimensional gaussian kernel */
double *
filter_build_gaussian2d(double *kernel, double d, int n)
{
  double *p, step, x, y, sum;
  int i, k;
  
  step = 6.0/((double)(n-1));
  p = kernel;
  sum = 0;

  y = -3.0;
  for (i = 0; i < n; i++)
    {
      x = -3.0;
      for (k = 0; k < n; k++)
        {
          *p++ = 1.0/(2.0*M_PI*d*d) * exp(-(x*x+y*y)/(2.0*d*d));
          sum += *(p-1);
          x += step;
        }
      y += step;
    }

  sum = 1.0/sum;
  p = kernel;
  for (i = 0; i < n*n; i++)
    *p++ *= sum;

  return kernel;
}

/* calculate laplacian of gaussian filter kernel. */
double *
filter_build_log(double *kernel, double d, int n)
{
  double *p, step, x, y, dd, xy;
  int i, k;
  
  step = 6.0/((double)(n-1));
  dd = d*d;
  p = kernel;

  y = -3.0;
  for (i = 0; i < n; i++)
    {
      x = -3.0;
      for (k = 0; k < n; k++)
        {
          xy = x*x+y*y;
          *p++ = -1.0/(M_PI*dd*dd)*(1.0-xy/(2.0*dd)) * exp(-xy/(2.0*dd));
          x += step;
        }
      y += step;
    }

  return kernel;
}

/* calculate difference of gaussians */
double *
filter_build_dog(double *kernel, double d0, double d1, int n)
{
  double *tmp = xmalloc(n*n*sizeof(double));
  double *p, *q;
  int size;
  
  filter_build_gaussian2d(kernel, d0, n);
  filter_build_gaussian2d(tmp, d1, n);

  p = kernel;
  q = tmp;
  size = n*n;

  while (size--)
    *p++ -= *q++;

  xfree(tmp);

  return kernel;
}
#endif
