/* im.h - image routines */

#ifndef IM_H
#define IM_H

#include <stdio.h>
#include "types.h"

/* image structure */
typedef struct
{
  int w, h;           /* dimensions */
  int channels;       /* number of channels */
  int *data;          /* pixel data */
} im_t;

/* image operations */
im_t *im_new(int, int, int);
void im_free(im_t *);
im_t *im_new_rotozoom(im_t *);
void im_rotozoom(im_t *, im_t *, double, double, double, double, double, double);
void im_crossfade(im_t *, im_t *, double);

/* read and write pnm files */
im_t *pnm_read(FILE *);
void pnm_write(FILE *, im_t *);

/* histogram routines */
double *histogram(im_t *, double *);
im_t *histogram_equalize(im_t *, im_t *);

/* image convolution */
im_t *im_convolve(im_t *, im_t *, int *, int, int);
im_t *im_convolve_separable(im_t *, im_t *, int *, int *, int, int);

int *convolve_x_3(int *, int *, int *, int, int, int);
int *convolve_x_5(int *, int *, int *, int, int, int);
int *convolve_y_3(int *, int *, int *, int, int, int);
int *convolve_y_5(int *, int *, int *, int, int, int);

im_t *im_convolve_x_3(im_t *, im_t *, int *);
im_t *im_convolve_x_5(im_t *, im_t *, int *);
im_t *im_convolve_y_3(im_t *, im_t *, int *);
im_t *im_convolve_y_5(im_t *, im_t *, int *);

im_t *im_convolve_separable_3x3(im_t *, im_t *, int *, int *);
im_t *im_convolve_separable_5x5(im_t *, im_t *, int *, int *);

/* filter kernels and methods */
int *filter_build_gaussian1d(int *, double, int);
int *filter_build_gaussian2d(int *, double, int);
int *filter_build_log(int *, double, int);
int *filter_build_dog(int *, double, double, int);

/* vector operations */
typedef double vec_t[2];

#define vec_mov(a,b) { (a)[0] = (b)[0]; (a)[1] = (b)[1]; }
#define vec_add(a,b) { (a)[0] += (b)[0]; (a)[1] += (b)[1]; }
#define vec_sub(a,b) { (a)[0] -= (b)[0]; (a)[1] -= (b)[1]; }
#define vec_mul(a,c) { (a)[0] *= c; (a)[1] *= c; }
#define vec_len(a) sqrt((a)[0]*(a)[0] + (a)[1]*(a)[1])
#define vec_dist(a,b) sqrt(((a)[0]-(b)[0])*((a)[0]-(b)[0]) + ((a)[1]-(b)[1])*((a)[1]-(b)[1]))
#define vec_sectx(a,b,y) ((a)[0] + ((y)-(a)[1])/((b)[1]-(a)[1])*((b)[0]-(a)[0]))
#define vec_secty(a,b,x) ((a)[1] + ((x)-(a)[0])/((b)[0]-(a)[0])*((b)[1]-(a)[1]))

#endif /* IM_H */
