/* input.c - human input for syncing etc */

#include <SDL.h>
#include "input.h"
#include "interpolate.h"

void input_smooth_variable(double *, double *, int);

double input[INPUT_NUM][INPUT_LEN]={{0}};
int input_i=INPUT_LEN-1;


void
input_record(void)
{
  Uint8 buttons;
  int x,y;

  SDL_PumpEvents();
  buttons = SDL_GetMouseState(&x,&y);

  input_i = (input_i+1)%INPUT_LEN;

  input_cur(0)=x;
  input_cur(1)=y;
  input_cur(2)=buttons&SDL_BUTTON(SDL_BUTTON_LEFT);
  input_cur(3)=buttons&SDL_BUTTON(SDL_BUTTON_RIGHT);
}

void
input_smooth(int step, double in[INPUT_NUM][INPUT_LEN])
{
  double a[INPUT_NUM], tmp[INPUT_NUM];
  int i,k,y2,y3;
  
  for (k=0; k<INPUT_NUM; k++)
    tmp[k] = in[k][0];
  
  for (i=0; i<INPUT_LEN; i++)
    {
      y2 = i+step; if (y2>=INPUT_LEN) y2 = INPUT_LEN-1;
      y3 = y2+step; if (y3>=INPUT_LEN) y3 = INPUT_LEN-1;
      
      for (k=0; k<2; k++)
        {
          interpolate_bspline_coeff(a,tmp[k],in[k][i],in[k][y2],in[k][y3]);
          tmp[k] = in[k][i];
          input_smooth_variable(&in[k][i],a,step);
        }
    }
}

void
input_smooth_variable(double *x, double *a, int n)
{
  double t,step;
  int i;
  
  step = 1.0/((double)n);
  for (i=0,t=0; i<n; i++,t+=step)
    x[i] = interpolate_cubic(a,t);
}

void
input_save(FILE *f, char *name)
{
  int i;
  
  fprintf(f, "input_t %s[%d] =\n{\n", name,INPUT_LEN);
  for (i=0; i<INPUT_LEN; i++)
    fprintf(f, "{%d,%d,%d},\n",(int)input[0][i],(int)input[1][i],(int)((input[2][i]>0?1:0)|(input[3][i]>0?2:0)));
  fprintf(f, "};\n");
}

void
input_convert(input_t *in, double out[INPUT_NUM][INPUT_LEN])
{
  int i;
  
  for (i=0; i<INPUT_LEN; i++)
    {
      out[0][i] = (double)in[i].x;
      out[1][i] = (double)in[i].y;
      out[2][i] = (double)(in[i].b&1?1.0:0.0);
      out[3][i] = (double)(in[i].b&2?1.0:0.0);
    }
}
