/* interpolate.h - interpolation functions */

#include <math.h>
#include "interpolate.h"

/* cosine interpolation. like linear, but smoothing the segments. */
double
interpolate_cosine(double y1, double y2, double t)
{
  double t2 = (1-cos(t*M_PI))/2;
  return y1*(1-t2) + y2*t2;
}

/* calculate coefficients for a simple cubic polynomial. consider control
   points equidistant (with distance 0.5) and make the transformation
   t = (x-x1) / (x2-x1). the coefficients can be derived by setting
   f(0) = y1, f(1) = y2, f'(0) = y2-y0, f'(1) = y3-y1. */
double *
interpolate_cubic_coeff(double *a, double y0, double y1, double y2, double y3)
{
  a[0] = y3-y2+y1-y0;
  a[1] = y0-y1-a[0];
  a[2] = y2-y0;
  a[3] = y1;

  return a;
}

/* calculate coefficients for a uniform cubic b-spline. */
double *
interpolate_bspline_coeff(double *a, double y0, double y1, double y2, double y3)
{
  a[0] = (-1*y0 + 3*y1 - 3*y2 + y3) / 6.0;
  a[1] = ( 3*y0 - 6*y1 + 3*y2     ) / 6.0;
  a[2] = (-3*y0        + 3*y2     ) / 6.0;
  a[3] = ( 1*y0 + 4*y1 + 1*y2     ) / 6.0;

  return a;
}

/* bilinear interpolation. */
double
interpolate_bilinear(double x, double y, double v0, double v1, double v2, double v3)
{
  double a, b;
  
  a = v0 + x*(v1-v0);
  b = v3 + x*(v2-v3);
  
  return a + y*(b-a);
}
