/* list.c - circularly linked list */

#include "list.h"
#include "xmalloc.h"

/* create a new list with a single element. */
list_t *
list_new(void *key)
{
  list_t *node = xmalloc(sizeof(list_t));
  
  node->key = key;
  node->prev = node;
  node->next = node;
  
  return node;
}

/* concatenate lists a and b. return the head of list a. */
list_t *
list_cat(list_t *a, list_t *b)
{
  list_t *tmp;
  
  /* if either is empty, the job is easy */
  if (!a || !b) return a ? a : b;
  
  /* otherwise we need to link them */
  tmp = a->prev;
  a->prev->next = b;
  a->prev = b->prev;
  b->prev->next = a;
  b->prev = tmp;

  return a;
}

/* add a new element to the head of the list. */
list_t *
list_add_head(list_t *head, void *key)
{
  list_t *node = list_new(key);
  return list_cat(node, head);
}

/* add a new element to the tail of the list. */
list_t *
list_add_tail(list_t *head, void *key)
{
  list_t *node = list_new(key);
  return list_cat(head, node);
}

/* remove an element from the list, return the list. */
list_t *
list_remove(list_t *head)
{
  list_t *next;
  
  if (!head) return NULL;

  next = head->next;
  head->prev->next = next;
  return next;
}

/* remove an element from the list and free it, return the list. */
list_t *
list_delete(list_t *head)
{
  list_t *list = list_remove(head);
  if (list == head) list = NULL;
  xfree(head);
  return list;
}
