/* pnm.c - read and write pnm files */

#include <stdlib.h>
#include <ctype.h>
#include "im.h"
#include "xmalloc.h"

void skip_whitespace(FILE *);


/* skip whitespace and comments */
void
skip_whitespace(FILE *f)
{
  int c;
  
  do
    {
      c = fgetc(f);
      if (c == '#')
        while ((c = fgetc(f)) != '\n');
    }
  while (isspace(c));
  ungetc(c,f);
}

/* read a pnm file and return an allocated image */
im_t *
pnm_read(FILE *f)
{
  im_t *im;
  int w, h, m, size, channels;
  uint8 id[2], *buf, *q;
  int *p;
  
  /* read header */
  id[0] = fgetc(f);
  id[1] = fgetc(f);

  if (id[0] != 'P')
    {
      fprintf(stderr, "error: cannot read a pnm file\n");
      exit(1);
    }

  switch (id[1])
    {
    case '5': channels = 1; break;
    case '6': channels = 3; break;
    
    default:
      fprintf(stderr, "error: cannot read a pnm file\n");
      exit(1);
    }

  skip_whitespace(f); fscanf(f, "%d", &w);
  skip_whitespace(f); fscanf(f, "%d", &h);
  skip_whitespace(f); fscanf(f, "%d", &m);
  fgetc(f);

  if (m > 255)
    {
      fprintf(stderr, "error: pnm maxval > 255 not supported\n");
      exit(1);
    }

  /* read pixel data */
  size = channels*w*h;
  buf = xmalloc(size);
  fread(buf, size, 1, f);

  /* convert to integers */
  im = im_new(w, h, channels);
  p = im->data;
  q = buf;
  while (size--)
    *p++ = *q++;

  xfree(buf);

  return im;
}

/* write a pnm file */
void
pnm_write(FILE *f, im_t *im)
{
  uint8 *buf, *p;
  int *q;
  int i, size;

  /* convert pixel data to bytes */
  i = size = im->w*im->h*im->channels;
  p = buf = xmalloc(size);
  q = im->data;
  while (i--)
    *p++ = (uint8)*q++;

  fprintf(f, im->channels == 3 ? "P6\n" : "P5\n");
  fprintf(f, "%d %d %d\n", im->w, im->h, 255);
  fwrite(buf, size, 1, f);

  xfree(buf);
}
